#!/usr/bin/python
import os
import json
import time
import urllib
import urllib2
from urllib2 import HTTPError


GA_COLLECT_SERVER = "https://www.google-analytics.com/collect"
GA_BATCH_SERVER = "https://www.google-analytics.com/batch"
Tracking_ID = "UA-59929653-29"
Filenames = ["/home/summary-2ab6fc51-f453-4c21-bd80-92f0d77468c2.json", "/home/vdaceip-bafed03f-ef3b-4a66-9db2-349b13d8e25e.json"]
#LOG_FILE = "/var/log/xdl/.ceip.log"
LOG_FILE = os.devnull



class UploadError(Exception):
    def __init__(self, error_code, value):
        self.error_code = error_code
        self.value = value
    def __str__(self):
        return "Error code: {0}, {1}".format(self.error_code, self.value)

def post(url, params, headers, data):
    if params is not None:
        url = url + "?" + urllib.urlencode(params)
    request = urllib2.Request(url, data, headers)
    try:
        res = urllib2.urlopen(request)
        return res
    except HTTPError, e:
        print e.reason
        raise UploadError(e.code, e.reason)

def upload_files(tid, url, *args):
    log = open(LOG_FILE, 'w+')
    log.write(time.ctime()+'\n')
    result = '' 
    if tid =='' or url =='' or args == []:
        return response
    log.write("tid: "+tid+", url: "+url+'\n')
    filenames = []
    filenames.extend(args)

    with open(LOG_FILE, "a") as log:
        log.write("filenames: "+", ".join(filenames)+'\n')
    
        # Distinguish vdaceip-* and sessionceip-* files
        #print filenames
        for filename in filenames:
            # resolve the file and upload the info
            #print filename+'\n'
            try:
                with open(filename, 'r') as f:
                    data = json.load(f)
                ret = ''
                if "vda" in filename:
                    ret = _upload_vda_file(tid,url,data)
                    result+=("\nFILE "+filename+" upload result: "+ret)
                elif "session" in filename:
                    ret = _upload_session_file(tid,url,data)            
                    result+=("\nFILE "+filename+" upload result:\n"+ret)
                elif "summary" in filename:
                    ret = _upload_summary_file(tid,url,data)
                    result+=("\nFILE "+filename+" upload result: "+ret)                 
                else:
                    result+=("\nFILE "+filename+": unexpected filename. Skip!")
            except UploadError as err:
                result+=("\nFILE "+filename+" UploadException: "+(str(err)))
            except IOError as err1:
                result+=("\nFILE "+filename+" IOException: "+(str(err1)))
            except Exception as err2:
                result+=("\nFILE "+filename+" Other Exception: "+(str(err2)))
            log.write('\n'+result+'\n')
    log.close()
    return result

def _upload_vda_file(tid, url, d):
    # Get system info from json
    #print "upload vda file"
    cid = d['machine_guid']
    cpu_cores = str(d['cpu_cores']) if d.has_key('cpu_cores') else ''
    cpu_frequency= str(d['cpu_frequency']) if d.has_key('cpu_frequency') else ''
    memory_size = str(d['memory_size']) if d.has_key('memory_size') else ''
    gpu_type = d['gpu_type'] if d.has_key('gpu_type') else ''
    os_name_version = d['os_name_version'] if d.has_key('os_name_version') else ''
    kernel_version = d['kernel_version'] if d.has_key('kernel_version') else ''
    ad_solution = d['ad_solution'] if d.has_key('ad_solution') else '(not set)'
    system_locale = d['system_locale'] if d.has_key('system_locale') else ''
    # Get vda info from json
    vda_version = d['vda_version'] if d.has_key('vda_version') else ''
    update_or_fresh_install = d['update_or_fresh_install'] if d.has_key('update_or_fresh_install') else '(not set)'
    vdi_mode = d['vdi_mode'] if d.has_key('vdi_mode') else '(not set)'
    hdx_3d_pro = d['hdx_3d_pro'] if d.has_key('hdx_3d_pro') else ''
    install_method = d['install_method'] if d.has_key('install_method') else '(not set)'
    sessions_maximum = str(d['sessions_maximum']) if d.has_key('sessions_maximum') else ''

    '''
    Data points             | GA format
    -------------------------------------------------------
    machine_guid            | cid
    machine_guid            | ec=system, ea=cid
    cpu_cores               | ec=system, ea=cpu_cores
    cpu_frequency           | ec=system, ea=cpu_frequency
    memory_size             | ec=system, ea=memory_size
    gpu_type                | ec=system, ea=gpu_type
    os_name_version         | cd1:os
    kernel_version          | ec=system, ea=kernel_version
    ad_solution             | cd2:ad
    system_locale           | ul
    vda_version             | cd3:lv
    update_or_fresh_install | cd4:it
    vdi_mode                | cd5:vdi
    hdx_3d_pro              | ec=vda, ea=hdx_3d_pro
    install_method          | cd6:im
    sessions_maximum        | ec=vda, ea=sessions_maximum
    '''

    # Construct payloads according to Google Analytics Measurement Protocol for different data points
    required_values = "v=1&tid="+tid+"&cid="+cid
    # Some data points are required to be sent as seperate dimensions
    common_values = "&cd1="+os_name_version+"&cd2="+ad_solution+"&ul="+system_locale+"&cd3="+vda_version+"&cd4="+update_or_fresh_install+"&cd5="+vdi_mode+"&cd6="+install_method
    # Construct list of events
    events = []
    events.append("&t=event&ec=system&ea=cid&el="+cid)
    events.append("&t=event&ec=system&ea=kernel_version&el="+kernel_version)
    events.append("&t=event&ec=system&ea=cpu_cores&el="+cpu_cores)
    events.append("&t=event&ec=system&ea=cpu_frequency&el="+cpu_frequency)
    events.append("&t=event&ec=system&ea=memory_size&el="+memory_size)
    events.append("&t=event&ec=system&ea=gpu_type&el="+gpu_type)
    events.append("&t=event&ec=vda&ea=hdx_3d_pro&el="+hdx_3d_pro)
    events.append("&t=event&ec=vda&ea=sessions_maximum&el="+sessions_maximum) 

    return _send_to_ga(url, required_values, events, common_values)


def _upload_summary_file(tid, url, d):
    #print "upload summary file"
    cid = d['machine_guid']
    session_launch = str(d['session_launch'])
    '''
    Data points             | GA format
    -------------------------------------------------------
    machine_guid            | cid
    session_launch          | ec=session, ea=session_launch, ev=d['session_launch']
    '''

    # Construct payloads according to Google Analytics Measurement Protocol for different data points
    required_values = "v=1&tid="+tid+"&cid="+cid
    # Some data points are required to be sent as seperate dimensions
    common_values = ""

    # Construct list of events
    events = [] 
    events.append("&t=event&ec=session&ea=session_launch&el=&ev="+session_launch)
    return _send_to_ga(url, required_values, events, common_values)


def _upload_session_file(tid, url, data):
    #print "upload session file"
    result = ''
    for d in data:
        # Get system info from json
        cid = d['machine_guid']
        session_key = d['session_key'] if d.has_key('session_key') else '(not set)' 
        os_name_version = d['os_name_version'] if d.has_key('os_name_version') else '' 
        vda_version = d['vda_version'] if d.has_key('vda_version') else ''       
        resource_type = d['resource_type'] if d.has_key('resource_type') else ''
        receiver_type = str(d['receiver_type']) if d.has_key('receiver_type') else ''
        receiver_version = d['receiver_version'] if d.has_key('receiver_version') else ''
        gfx_provider_type = d['gfx_provider_type'] if d.has_key('gfx_provider_type') else ''
        grahpic_video_codec_type = d['grahpic_video_codec_type'] if d.has_key('grahpic_video_codec_type') else ''
        credentials_type = str(d['credentials_type']) if d.has_key('credentials_type') else ''
        printing_count = str(d['printing_count']) if d.has_key('printing_count') else '0'
        usb_redirecting_count = str(d['usb_redirecting_count']) if d.has_key('usb_redirecting_count') else '0'
        shadow_count = str(d['shadow_count']) if d.has_key('shadow_count') else '0'
        scard_redirecting_count = str(d['scard_redirecting_count']) if d.has_key('scard_redirecting_count') else '0'
        ctxism_select = d['ctxism_select'] if d.has_key('ctxism_select') else ''
        # time  unit is second
        session_duration_time = str(d['session_duration_time']) if d.has_key('session_duration_time') else ''
        active_session_time = map(str,d['active_session_time']) if d.has_key('active_session_time') else ''

        '''
        Data points             | GA format
        -------------------------------------------------------
        os_name_version         | cd1:os
        vda_version             | cd3:lv
        session_key             | cd7:sk
        resource_type           | an
        receiver_type           | ec=session, ea=receiver_type
        receiver_version        | ec=session, ea=receiver_version
        gfx_provider_type       | ec=session, ea=gfx_provider_type
        grahpic_video_codec_type| ec=session, ea=grahpic_video_codec_type
        credentials_type        | ec=session, ea=credentials_type
        printing_count          | ec=session, ea=printing_count
        usb_redirecting_count   | ec=session, ea=usb_redirecting_count
        shadow_count            | ec=session, ea=shadow_count
        ctxism_select           | ec=session, ea=ctxism_select
        session_duration_time   | utc=session, utv=session_duration_time
        active_session_time     | utc=session, utv=active_session_time

        '''


        # Construct payloads according to Google Analytics Measurement Protocol for different data points
        required_values = "v=1&tid="+tid+"&cid="+cid
        # Some data points are required to be sent as seperate dimensions
        common_values = "&cd1="+os_name_version+"&cd3="+vda_version+"&cd7="+session_key+"&an="+resource_type
        # Construct list of event hits
        events = []
        events.append("&t=event&ec=session&ea=receiver_type&el="+receiver_type)
        events.append("&t=event&ec=session&ea=receiver_version&el="+receiver_version)
        events.append("&t=event&ec=session&ea=gfx_provider_type&el="+gfx_provider_type)         
        events.append("&t=event&ec=session&ea=grahpic_video_codec_type&el="+grahpic_video_codec_type)
        events.append("&t=event&ec=session&ea=credentials_type&el="+credentials_type)
        events.append("&t=event&ec=session&ea=printing_count&el="+printing_count)
        events.append("&t=event&ec=session&ea=usb_redirecting_count&el="+usb_redirecting_count)
        events.append("&t=event&ec=session&ea=shadow_count&el="+shadow_count)           
        events.append("&t=event&ec=session&ea=scard_redirecting_count&el="+scard_redirecting_count)     
        events.append("&t=event&ec=session&ea=ctxism_select&el="+ctxism_select)     
            
        # Construct list of timing hits
        timings = []
        timings.append("&t=timing&utc=session&utv=session_duration_time&utl=&utt="+session_duration_time)
        # active_session_time
        timings.append("&t=timing&utc=session&utv=active_session_time&utl=min&utt="+active_session_time[0])
        timings.append("&t=timing&utc=session&utv=active_session_time&utl=max&utt="+active_session_time[1])
        timings.append("&t=timing&utc=session&utv=active_session_time&utl=sum&utt="+active_session_time[2])
        timings.append("&t=timing&utc=session&utv=active_session_time&utl=count&utt="+active_session_time[3])
        timings.append("&t=timing&utc=session&utv=active_session_time&utl=avg&utt="+active_session_time[4])

        ret = _send_to_ga(url, required_values, events+timings, common_values)
        result+=("    SESSION "+session_key+": "+ret+'.\n')
    return result
    
def _send_to_ga(url, required_values, ets, common_values):
    '''
    The Measurement Protocol will return a 2xx status code if the HTTP request was received. 
    The Measurement Protocol does not return an error code if the payload data was malformed, or if the data in the payload was incorrect or was not processed by Google Analytics. 
    If you do not get a 2xx status code, you should NOT retry the request.
    '''
    response = 'success'
    with open(LOG_FILE, "a") as log:
        for et in ets:
            p = required_values+et+common_values
            r = post(url, None, {}, data = p) 
            log.write("Send:  "+p+'\n')
            #log.write("Text: "+r.text+'\n'))
            #log.write("Status_code: "+r.status_code+'\n'))
            #log.write("Reason: "+r.reason+'\n'))

        #TODO: use GA batch to send multiple payloads at one time

    return response

if __name__ =="__main__":
    upload_files(Tracking_ID,GA_COLLECT_SERVER,*Filenames)
