const Lang = imports.lang;
const Gio = imports.gi.Gio;
const IBusManager = imports.misc.ibusManager;
const InputSourceManager = imports.ui.status.keyboard;

const CtxKeyboard = new Lang.Class({
    Name: 'CtxKeyboard',
    _DESKTOP_INPUT_SOURCES_SCHEMA: 'org.gnome.desktop.input-sources',
    _KEY_CURRENT: 'current',
    _init: function() {
        this._started = false;
        this._ibusReady = false;
        this._currentActived = -1;
        this._settings = new Gio.Settings({ schema_id: this._DESKTOP_INPUT_SOURCES_SCHEMA });
        this._settings.connect('changed::' + this._KEY_CURRENT, Lang.bind(this, this._rcvCurrentChanged));
        this._inputSourceManager = InputSourceManager.getInputSourceManager();
        this._ibusManager = IBusManager.getIBusManager();
        this._ibusManager.connect('ready', Lang.bind(this, this._ibusReadyCallback));
    },
    _rcvCurrentChanged: function() {
        let current = this._getCurrent();
        let sources = this._inputSourceManager.inputSources;
        let idx = current.deep_unpack();
        this._currentActived = idx;
        if (!this._ibusReady) {
            global.log('citrix extension: recv keyboard idx: ' + idx + ', wait for ibus ready');
            return;
        }
        global.log('citrix extension: set current keyboard idx: ' + idx);
        let currentSource = sources[idx];
        currentSource.activate(true);
    },

    _ibusReadyCallback: function(im, ready) {
        if (this._ibusReady == ready)
            return;
        this._ibusReady = ready;
        if (!this._ibusReady) {
            return;
        }
        if (this._currentActived == -1) {
            let current = this._getCurrent();
            this._currentActived = current.deep_unpack();
            global.log('citrix extension: the changed message is missed');
        }
        let sources = this._inputSourceManager.inputSources;
        global.log('citrix extension: change current keyboard idx: ' + this._currentActived + ', after ibus ready');
        let currentSource = sources[this._currentActived];
        currentSource.activate(true);
    },

    _getCurrent: function() {
        return this._settings.get_value(this._KEY_CURRENT);
    },

    get started() {
        return this._started;
    },

    set started(value) {
        this._started = value;
    }
});

let module;

function init() {
    global.log("citrix extension: start to init citrix extension");
    module = new CtxKeyboard();
}

function enable() {
    global.log("citrix extension: enable");
    module.started = true;
}

function disable() {
    global.log("citrix extension: disable");
    module.started = false;
}