#!/bin/sh

function ldconfig_update {
    function inner {
        ldconfig /opt/Citrix/VDA/lib64 || return 1
        #ldconfig /opt/Citrix/VDA/lib32 || return 1
    }
    cmdlog_exec inner $FUNCNAME "update ldconfig"
}

function postgresql_start {
    function inner {
        systemctl start postgresql || return 1
    }
    cmdlog_exec inner $FUNCNAME "start PostgreSQL"
}

function postgresql_restart {
    function inner {
        systemctl restart postgresql || return 1
    }
    cmdlog_exec inner $FUNCNAME "restart PostgreSQL"
}

function service_start {
    function inner {
        systemctl daemon-reload || return 1
        if [ -f /etc/systemd/system/ctxlogd.service ]; then
            systemctl start ctxlogd || return 1
        fi
		
        systemctl start ctxhdx || return 1
        systemctl start ctxvda || return 1
       
        systemctl start ctxcups
        systemctl start cups
        systemctl start ctxvhcid
        systemctl start ctxusbsd
        systemctl start ctxpolicyd || return 1
        systemctl start ctxceip
        systemctl start ctxcdm

    }
    cmdlog_exec inner $FUNCNAME "start $short_name"
}

function smartcard_start {
    if [ -f /etc/systemd/system/ctxscardsd.service ]; then
        result=`/opt/Citrix/VDA/bin/ctxreg dump | grep SmartCard | awk {'print $9'}`

        if [[ $result =~ "0x00000000" ]]; then
            systemctl stop ctxscardsd || return 1
        else
            systemctl start ctxscardsd || return 1
        fi
    fi
}

function service_stop {
    function inner {
        systemctl daemon-reload || return 1
        pkill -f /opt/Citrix/VDA/bin/ctxfm
        if [ -f /etc/systemd/system/ctxscardsd.service ]; then
             systemctl stop ctxscardsd || return 1
        fi
		
        systemctl stop ctxcdm
        systemctl stop ctxcups
        systemctl stop cups
        systemctl stop ctxusbsd
        systemctl stop ctxvhcid
        systemctl stop ctxceip
 
        systemctl stop ctxpolicyd || return 1
        systemctl stop ctxvda || return 1
        systemctl stop ctxhdx || return 1

        if [ -f /etc/systemd/system/ctxlogd.service ]; then
            systemctl stop ctxlogd || return 1
        fi

    }
    cmdlog_exec inner $FUNCNAME "stop $short_name"
}

function service_kill {
    function inner {
        systemctl is-active -q ctxvda && systemctl kill ctxvda
        systemctl is-active -q ctxhdx && systemctl kill ctxhdx
        if [ -f /etc/systemd/system/ctxlogd.service ]; then
            systemctl is-active -q ctxlogd && systemctl kill ctxlogd
        fi
        if [ -f /etc/systemd/system/ad_join.service ]; then
            systemctl is-active -q ad_join && systemctl kill ad_join
        fi
		
		
        return 0
    }
    cmdlog_exec inner $FUNCNAME "kill $short_name"
}

function service_register {
    function inner {
        systemctl daemon-reload || return 1
        if [ -f /etc/systemd/system/ctxlogd.service ]; then
            systemctl enable ctxlogd || return 1
        fi
		
        systemctl enable ctxhdx || return 1
        systemctl enable ctxvda || return 1
        
        systemctl enable ctxpolicyd || return 1
        systemctl enable ctxcups
        systemctl enable cups
        systemctl enable ctxcdm
        systemctl enable ctxceip
        systemctl enable ctxvhcid
        systemctl enable ctxusbsd

        if [ -f /etc/systemd/system/ctxscardsd.service ]; then
             systemctl enable ctxscardsd || return 1
        fi
    }
    cmdlog_exec inner $FUNCNAME "register $short_name services"
}

function service_unregister {
    function inner {
        systemctl daemon-reload || return 1
        systemctl disable ctxcups
        systemctl disable cups
		
        systemctl disable ctxcdm
        systemctl disable ctxusbsd
        systemctl disable ctxvhcid
        systemctl disable ctxceip      

        systemctl disable ctxvda || return 1
        systemctl disable ctxhdx || return 1

        systemctl disable ctxpolicyd || return 1
        if [ -f /etc/systemd/system/ctxlogd.service ]; then
            systemctl disable ctxlogd || return 1
        fi

        if [ -f /etc/systemd/system/ad_join.service ]; then
            systemctl disable ad_join || return 1
        fi

        if [ -f /etc/systemd/system/ctxscardsd.service ]; then
             systemctl disable ctxscardsd || return 1
        fi

    }
    cmdlog_exec inner $FUNCNAME "unregister $short_name services"
}

function firewall_add_rules {
    function inner {
        if firewall-cmd --state; then
            firewall-cmd --permanent --zone=public --add-port=$1/tcp || return 1
            firewall-cmd --permanent --zone=public --add-port=1494/tcp || return 1
            firewall-cmd --permanent --zone=public --add-port=2598/tcp || return 1
            firewall-cmd --permanent --zone=public --add-port=1494/udp || return 1
            firewall-cmd --permanent --zone=public --add-port=2598/udp || return 1
            firewall-cmd --permanent --zone=public --add-port=8008/tcp || return 1
            firewall-cmd --permanent --zone=public --add-port=6001-6099/tcp || return 1
        else
            return 0
        fi
    }
    cmdlog_exec inner $FUNCNAME "add firewall rules for $short_name" $1
}

function firewall_update_rules {
    function inner {
        if firewall-cmd --state; then
            firewall-cmd --permanent --zone=public --add-port=2598/tcp || return 1
            firewall-cmd --permanent --zone=public --add-port=1494/udp || return 1
            firewall-cmd --permanent --zone=public --add-port=2598/udp || return 1
            firewall-cmd --permanent --zone=public --add-port=8008/tcp || return 1
            firewall-cmd --permanent --zone=public --add-port=6001-6099/tcp || return 1
        else
            return 0
        fi
    }
    cmdlog_exec inner $FUNCNAME "update firewall rules for $short_name" $1
}

function firewall_remove_rules {
    function inner {
        if firewall-cmd --state; then
            export vda_port=$(printf "%d" $(/opt/Citrix/VDA/bin/ctxreg read -k "HKLM\Software\Citrix\VirtualDesktopAgent" -v "ControllerRegistrarPort")) || return 1
            firewall-cmd --permanent --zone=public --remove-port=$vda_port/tcp || return 1
            firewall-cmd --permanent --zone=public --remove-port=1494/tcp || return 1
            firewall-cmd --permanent --zone=public --remove-port=2598/tcp || return 1
            firewall-cmd --permanent --zone=public --remove-port=1494/udp || return 1
            firewall-cmd --permanent --zone=public --remove-port=2598/udp || return 1
            firewall-cmd --permanent --zone=public --remove-port=8008/tcp || return 1
            firewall-cmd --permanent --zone=public --remove-port=6001-6099/tcp || return 1
        else
            return 0
        fi
    }
    cmdlog_exec inner $FUNCNAME "remove firewall rules for $short_name"
}

function firewall_try_restart {
    function inner {
        if firewall-cmd --state; then
            firewall-cmd --reload || return 1
        else
            return 0
        fi
    }
    cmdlog_exec inner $FUNCNAME "restart the firewall"
}

function ctxXrandrhook {
    BEGIN_PATTERN="# Citrix configuration begin"
    MAIN_CONTENT="export LD_PRELOAD='/usr/\$LIB/libctxXrandrhook.so'"
    END_PATTERN="# Citrix congiguration end"
    XCLIENTS="/etc/X11/xinit/Xclients"
    XCLIENTS_ORIG="/etc/X11/xinit/Xclients.orig"
    [ "$#" -ne "1" ] && return 1
    case $1 in
        "configure")
            if [ -f $XCLIENTS ]; then
                [[ ! -f $XCLIENTS_ORIG ]] && cp $XCLIENTS $XCLIENTS_ORIG
                sed -i -e "/$BEGIN_PATTERN/,/$END_PATTERN/d" $XCLIENTS
                awk -v "txt=$BEGIN_PATTERN\n$MAIN_CONTENT\n$END_PATTERN" '/^[^#]/ && !x {print txt; x=1} 1' "$XCLIENTS" > "${XCLIENTS}.tmp" && mv -f "${XCLIENTS}.tmp" "$XCLIENTS" && chmod a+x+r "$XCLIENTS"
            fi
            ;;
        "restore")
            sed -i -e "/$BEGIN_PATTERN/,/$END_PATTERN/d" $XCLIENTS
            ;;
        *)
            ;;
    esac
}
function pcsclibhook {
    PCSC_LIB_NAME="libpcsclite.so.1.0.0"
    PCSC_HOOK_NAME="libctxpcsc.so"
    PCSC_LIB_LINK="libpcsclite.so.1"
    PCSC_LIB_DIR="/usr/lib64"
    PCSC_BAK_DIR="/var/xdl"
    PCSC_HOOK_DIR="/opt/Citrix/VDA/lib64"
    
    [ "$#" -ne "1" ] && return 1
    case $1 in
        "configure")
            if [ -e $PCSC_LIB_DIR/$PCSC_LIB_NAME ]; then
                mv $PCSC_LIB_DIR/$PCSC_LIB_NAME $PCSC_BAK_DIR/
                ldconfig
                
                if [ -e $PCSC_LIB_DIR/$PCSC_LIB_LINK ]; then
                    rm -f $PCSC_LIB_DIR/$PCSC_LIB_LINK
                fi
                
                ln $PCSC_HOOK_DIR/$PCSC_HOOK_NAME -s $PCSC_LIB_DIR/$PCSC_LIB_LINK
                
            fi
            ;;
        "restore")
            if [ -e $PCSC_BAK_DIR/$PCSC_LIB_NAME ]; then
                mv $PCSC_BAK_DIR/$PCSC_LIB_NAME $PCSC_LIB_DIR/
                ldconfig
            fi
            ;;
        *)
            ;;
    esac

}

function disable_gnome_smartcard_kit {
    ESC_FILE="/etc/xdg/autostart/esc.desktop"
    SCARD_FILE="/etc/xdg/autostart/org.gnome.SettingsDaemon.Smartcard.desktop"
    
    [ "$#" -ne "1" ] && return 1
    case $1 in
        "configure")
            if [ -f $ESC_FILE ]; then
                sed -i '$a\Hidden=true' $ESC_FILE
            fi
            
            if [ -f $SCARD_FILE ]; then
                sed -i '$a\Hidden=true' $SCARD_FILE
            fi
            ;;
        "restore")
            if [ -f $ESC_FILE ]; then
                sed -i '/^Hidden=true/d' $ESC_FILE
            fi
            
            if [ -f $SCARD_FILE ]; then
                sed -i '/^Hidden=true/d' $SCARD_FILE
            fi
            ;;
        *)
            ;;
    esac

}

function link_pam_pkcs11 {
    SRC_FILE="/lib64/security/pam_pkcs11.so"
    LINK_FILE="/usr/lib64/pam_pkcs11.so"

    [ "$#" -ne "1" ] && return 1
    case $1 in
        "configure")
            if [ -f $SRC_FILE ]; then
                if [ ! -e $LINK_FILE ]; then
                    ln $SRC_FILE -s $LINK_FILE
                fi
            fi
            ;;
        "restore")
            if [ -L $LINK_FILE ]; then
                rm -f $LINK_FILE
            fi
            ;;
        *)
            ;;
    esac

}

function desktop_environment_configure {
    function inner {
        ctxXrandrhook configure
        pcsclibhook configure
        disable_gnome_smartcard_kit configure
        link_pam_pkcs11 configure
        cp -f /var/xdl/10-ctx-vhcd-usb-device.rules /etc/udev/rules.d
        cp -f /var/xdl/10-ctx-vhcd-usb-mount.rules /etc/udev/rules.d
#        cp -f /usr/lib/systemd/system/systemd-udevd.service /etc/systemd/system/systemd-udevd.service
#        sed -i "s/MountFlags/#MountFlags/g" /etc/systemd/system/systemd-udevd.service
#        systemctl daemon-reload
#        systemctl restart systemd-udevd.service
        cp -f /var/xdl/10-ctxhdx-disable-power-management.rules /usr/share/polkit-1/rules.d || return 1
    }
    cmdlog_exec inner $FUNCNAME "desktop environment configured"
}

function desktop_environment_restore {
    function inner {
        ctxXrandrhook restore
        pcsclibhook restore
        disable_gnome_smartcard_kit restore
        link_pam_pkcs11 restore
        rm -f /etc/udev/rules.d/10-ctx-vhcd-usb-device.rules
        rm -f /etc/udev/rules.d/10-ctx-vhcd-usb-mount.rules
#        rm -f /etc/systemd/system/systemd-udevd.service
#        systemctl daemon-reload
#        systemctl restart systemd-udevd.service
        rm -f /usr/share/polkit-1/rules.d/10-ctxhdx-disable-power-management.rules || return 1
    }
    cmdlog_exec inner $FUNCNAME "desktop environment restored"
}

function remove_mcs_config {
    function inner {
        dbus_service_file_path="/lib/systemd/system/dbus.service"
        sed -i -e '/^ExecStartPre=.*$/ d' "$dbus_service_file_path"
    }
    cmdlog_exec inner $FUNCNAME "Remove mcs configurations"
}
